package net.wanji.opt.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import net.wanji.common.enums.*;
import net.wanji.common.utils.tool.CrossUtil;
import net.wanji.databus.bo.CrossIdBO;
import net.wanji.databus.dao.entity.*;
import net.wanji.databus.dao.mapper.*;
import net.wanji.databus.po.*;
import net.wanji.databus.vo.AbnormalCrossListVO;
import net.wanji.databus.vo.GreenwaveListVO;
import net.wanji.opt.bo.*;
import net.wanji.opt.config.DirectionMappingsConfig;
import net.wanji.opt.dao.mapper.CrossSchemeOptLogMapper;
import net.wanji.opt.dao.mapper.strategy.SceneMapper;
import net.wanji.opt.dao.mapper.strategy.SceneStrategyIdeaMapper;
import net.wanji.opt.dao.mapper.strategy.SceneStrategyMapper;
import net.wanji.opt.dao.mapper.strategy.StrategyMapper;
import net.wanji.opt.dao.mapper.trend.EventAlarmMapper;
import net.wanji.opt.dto.CrossLaneSnapshotDataDTO;
import net.wanji.opt.dto.LineSchemeDTO;
import net.wanji.opt.dto.trend.AbnormalCrossListDTO;
import net.wanji.opt.dto.trend.GreenwaveListDTO;
import net.wanji.opt.po.base.CrossSchemeOptLogPO;
import net.wanji.opt.po.strategy.ScenePO;
import net.wanji.opt.po.strategy.StrategyPO;
import net.wanji.opt.po.trend.EventAlarmPO;
import net.wanji.opt.service.TrendService;
import net.wanji.opt.service.es.LaneSnapshotDataQueryService;
import net.wanji.opt.vo.*;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.*;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.function.Function;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * @author Kent HAN
 * @date 2023/2/8 10:31
 */
@Service
public class TrendServiceImpl implements TrendService {
    private final GreenwaveInfoMapper greenwaveInfoMapper;
    private final BaseCrossInfoMapper baseCrossInfoMapper;
    private final CrossDataRealtimeMapper crossDataRealtimeMapper;
    private final CrossTurnDataRealtimeMapper crossTurnDataRealtimeMapper;
    private final CrossDirDataHistMapper crossDirDataHistMapper;
    private final EventAlarmMapper eventAlarmMapper;
    private final RidInfoMapper ridInfoMapper;
    private final BaseCrossDirInfoMapper baseCrossDirInfoMapper;
    private final CrossSchemeOptLogMapper crossSchemeOptLogMapper;
    private final GreenwaveCrossMapper greenwaveCrossMapper;
    private final CrossDirDataRealtimeMapper crossDirDataRealtimeMapper;
    private final GreenwaveHistMapper greenwaveHistMapper;
    private final GreenwaveRealtimeMapper greenwaveRealtimeMapper;
    private final StrategyMapper strategyMapper;
    private final BaseCrossSchedulesMapper baseCrossSchedulesMapper;
    private final BaseCrossSchedulesPlanMapper baseCrossSchedulesPlanMapper;
    private final BaseCrossSectionMapper baseCrossSectionMapper;
    private final BaseCrossSchemeMapper baseCrossSchemeMapper;
    private final BaseCrossPhaseMapper baseCrossPhaseMapper;
    private final BaseCrossPhaseLightsMapper crossPhaseLightsMapper;
    private final BaseCrossLaneLightsMapper baseCrossLaneLightsMapper;
    private final CrossBaseLaneInfoMapper crossBaseLaneInfoMapper;
    private final CrossTurnDataHistMapper crossTurnDataHistMapper;
    private final GreenwaveSceneMapper greenwaveSceneMapper;
    private final SceneStrategyMapper sceneStrategyMapper;
    private final SceneMapper sceneMapper;
    private final BaseCrossPlanMapper baseCrossPlanMapper;
    private final MainlineEvaluateServiceImpl mainlineEvaluateServiceImpl;
    private final CrossLaneDataHistMapper crossLaneDataHistMapper;
    private final LaneSnapshotDataQueryService laneSnapshotDataQueryService;
    private final DirectionMappingsConfig directionMappingsConfig;

    // 用于计算路口状态,key为方向，value为状态
    private Map<Integer, Integer> preStatus = new HashMap<Integer, Integer>() {{
        put(1, 1);
        put(2, 1);
        put(3, 1);
        put(4, 1);
        put(5, 1);
        put(6, 1);
        put(7, 1);
        put(8, 1);
    }};

    // 表格实时推送开始时间
    private Date tableRealTimeFirstDate;
    // 表格周期推送开始时间
    private Date tableCycleFirstDate;

    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    public TrendServiceImpl(GreenwaveInfoMapper greenwaveInfoMapper, BaseCrossInfoMapper baseCrossInfoMapper,
                            CrossDataRealtimeMapper crossDataRealtimeMapper,
                            CrossTurnDataRealtimeMapper crossTurnDataRealtimeMapper,
                            CrossDirDataHistMapper crossDirDataHistMapper, EventAlarmMapper eventAlarmMapper,
                            RidInfoMapper ridInfoMapper, BaseCrossDirInfoMapper baseCrossDirInfoMapper,
                            CrossSchemeOptLogMapper crossSchemeOptLogMapper, GreenwaveCrossMapper greenwaveCrossMapper, CrossDirDataRealtimeMapper crossDirDataRealtimeMapper, GreenwaveHistMapper greenwaveHistMapper, GreenwaveRealtimeMapper greenwaveRealtimeMapper, SceneStrategyIdeaMapper strategyIdeaMapper, StrategyMapper strategyMapper, BaseCrossSchedulesMapper baseCrossSchedulesMapper, BaseCrossSchedulesPlanMapper baseCrossSchedulesPlanMapper, BaseCrossSectionMapper baseCrossSectionMapper, BaseCrossSchemeMapper crossSchemeMapper, BaseCrossSchemeMapper baseCrossSchemeMapper, BaseCrossPhaseMapper baseCrossPhaseMapper, BaseCrossPhaseLightsMapper crossPhaseLightsMapper, BaseCrossLaneLightsMapper baseCrossLaneLightsMapper, CrossBaseLaneInfoMapper crossBaseLaneInfoMapper, CrossTurnDataHistMapper crossTurnDataHistMapper, @Qualifier("greenwaveSceneMapper") GreenwaveSceneMapper greenwaveSceneMapper, @Qualifier("sceneStrategyMapper") SceneStrategyMapper sceneStrategyMapper, @Qualifier("sceneMapper") SceneMapper sceneMapper, @Qualifier("baseCrossPlanMapper") BaseCrossPlanMapper baseCrossPlanMapper, MainlineEvaluateServiceImpl mainlineEvaluateServiceImpl, CrossLaneDataHistMapper crossLaneDataHistMapper, LaneSnapshotDataQueryService laneSnapshotDataQueryService, DirectionMappingsConfig directionMappingsConfig) {
        this.greenwaveInfoMapper = greenwaveInfoMapper;
        this.baseCrossInfoMapper = baseCrossInfoMapper;
        this.crossDataRealtimeMapper = crossDataRealtimeMapper;
        this.crossTurnDataRealtimeMapper = crossTurnDataRealtimeMapper;
        this.crossDirDataHistMapper = crossDirDataHistMapper;
        this.eventAlarmMapper = eventAlarmMapper;
        this.ridInfoMapper = ridInfoMapper;
        this.baseCrossDirInfoMapper = baseCrossDirInfoMapper;
        this.crossSchemeOptLogMapper = crossSchemeOptLogMapper;
        this.greenwaveCrossMapper = greenwaveCrossMapper;
        this.crossDirDataRealtimeMapper = crossDirDataRealtimeMapper;
        this.greenwaveHistMapper = greenwaveHistMapper;
        this.greenwaveRealtimeMapper = greenwaveRealtimeMapper;
        this.strategyMapper = strategyMapper;
        this.baseCrossSchedulesMapper = baseCrossSchedulesMapper;
        this.baseCrossSchedulesPlanMapper = baseCrossSchedulesPlanMapper;
        this.baseCrossSectionMapper = baseCrossSectionMapper;
        this.baseCrossSchemeMapper = baseCrossSchemeMapper;
        this.baseCrossPhaseMapper = baseCrossPhaseMapper;
        this.crossPhaseLightsMapper = crossPhaseLightsMapper;
        this.baseCrossLaneLightsMapper = baseCrossLaneLightsMapper;
        this.crossBaseLaneInfoMapper = crossBaseLaneInfoMapper;
        this.crossTurnDataHistMapper = crossTurnDataHistMapper;
        this.greenwaveSceneMapper = greenwaveSceneMapper;
        this.sceneStrategyMapper = sceneStrategyMapper;
        this.sceneMapper = sceneMapper;
        this.baseCrossPlanMapper = baseCrossPlanMapper;
        this.mainlineEvaluateServiceImpl = mainlineEvaluateServiceImpl;
        this.crossLaneDataHistMapper = crossLaneDataHistMapper;
        this.laneSnapshotDataQueryService = laneSnapshotDataQueryService;
        this.directionMappingsConfig = directionMappingsConfig;
    }

    @Override
    public List<GreenwaveListVO> greenwaveList(GreenwaveListDTO greenwaveListDTO) {
        Integer status = greenwaveListDTO.getStatus();
        String name = greenwaveListDTO.getName();
        Integer type = greenwaveListDTO.getType();
        List<GreenwaveListVO> greenwaveListVOList = greenwaveInfoMapper.listGreenwave(status, name, type);
        // 3、4都算拥堵
        if (status != null && status == 3) {
            List<GreenwaveListVO> extraList = greenwaveInfoMapper.listGreenwave(4, name, type);
            greenwaveListVOList.addAll(extraList);
        }
        changeStatus(greenwaveListVOList);
        for (GreenwaveListVO greenwaveListVO : greenwaveListVOList) {
            // 计算持续时间
            buildDuration(greenwaveListVO);
            // 构造路口列表
            Integer greenwaveId = greenwaveListVO.getId();
            List<GreenwaveListVO.CrossListElement> crossList = buildCrossList(greenwaveId);
            greenwaveListVO.setCrossList(crossList);
        }
        return greenwaveListVOList;
    }

    @Override
    public GreenwaveStats buildGreenwaveStats(List<GreenwaveListVO> greenwaveListVOList) {
        GreenwaveStats greenwaveStats = new GreenwaveStats();
        Integer smooth = greenwaveStats.getSmooth();
        Integer slow = greenwaveStats.getSlow();
        Integer congestion = greenwaveStats.getCongestion();
        for (GreenwaveListVO greenwaveListVO : greenwaveListVOList) {
            Integer realtimeStatus = greenwaveListVO.getRealtimeStatus();
            if (realtimeStatus == 1) {
                smooth += 1;
            } else if (realtimeStatus == 2) {
                slow += 1;
            } else if (realtimeStatus == 3) {
                congestion += 1;
            }
        }
        greenwaveStats.setSmooth(smooth);
        greenwaveStats.setSlow(slow);
        greenwaveStats.setCongestion(congestion);
        return greenwaveStats;
    }

    @Override
    public AbnormalCrossVO abnormalCrossList(AbnormalCrossListDTO abnormalCrossListDTO) {
        AbnormalCrossVO abnormalCrossVO = new AbnormalCrossVO();
        // 构建问题路口列表
        Integer status = abnormalCrossListDTO.getStatus();
        String name = abnormalCrossListDTO.getName();
        Integer type = abnormalCrossListDTO.getType();
        List<AbnormalCrossListVO> abnormalCrossListVOList =
                crossDataRealtimeMapper.selectAbnormalCross(status, name, type);
        // 添加优化状态
        for (AbnormalCrossListVO abnormalCrossListVO : abnormalCrossListVOList) {
            abnormalCrossListVO.setOptStatus(2); // 2未优化
            String crossId = abnormalCrossListVO.getId();
            // 查询优化日志
            List<CrossSchemeOptLogPO> crossSchemeOptLogPOS = crossSchemeOptLogMapper.selectMaxByCrossId(crossId);
            if (CollectionUtil.isNotEmpty(crossSchemeOptLogPOS)) {
                CrossSchemeOptLogPO crossSchemeOptLogPO = crossSchemeOptLogPOS.get(0);
                Date endTime = crossSchemeOptLogPO.getEndTime();
                if (ObjectUtil.isEmpty(endTime)) {
                    abnormalCrossListVO.setOptStatus(1); // 1优化中
                }
            }
        }
        // 坐标格式转换
        for (AbnormalCrossListVO abnormalCrossListVO : abnormalCrossListVOList) {
            String locationStr = abnormalCrossListVO.getLocationStr();
            double[] lonLat = CrossUtil.getLonLat(locationStr);
            List<Double> location = new ArrayList<>();
            location.add(lonLat[0]);
            location.add(lonLat[1]);
            abnormalCrossListVO.setLocation(location);
            // 关联路口信息
            Integer realtimeStatus = abnormalCrossListVO.getRealtimeStatus();
            String endCrossId = abnormalCrossListVO.getId();
            if (Objects.equals(realtimeStatus, CrossStatusEnum.SPILLOVER.getCode())) {
                // 溢出只返回溢出方向的关联路口
                String spilloverDirs = abnormalCrossListVO.getSpilloverDirs();
                String[] split = spilloverDirs.split(",");
                List<Integer> spilloverDirList = new ArrayList<>();
                for (String s : split) {
                    spilloverDirList.add(Integer.parseInt(s));
                }
                buildRidData(abnormalCrossListVO, endCrossId, spilloverDirList);
            } else if (Objects.equals(realtimeStatus, CrossStatusEnum.CONGESTION.getCode())){
                // 拥堵返回所有方向关联路口
                List<Integer> inDirs = baseCrossDirInfoMapper.selectInDirsByCrossId(endCrossId);
                buildRidData(abnormalCrossListVO, endCrossId, inDirs);
            }
        }
        CrossComparator crossComparator = new CrossComparator();
        abnormalCrossListVOList.sort(crossComparator);
        abnormalCrossVO.setAbnormalCrossList(abnormalCrossListVOList);
        // 构造统计信息
        List<AbnormalCrossListVO> listForStats =
                crossDataRealtimeMapper.selectAbnormalCross(null, null, null);
        buildAbnormalCrossStats(abnormalCrossVO, listForStats);

        return abnormalCrossVO;
    }

    private static class CrossComparator implements Comparator<AbnormalCrossListVO> {
        @Override
        public int compare(AbnormalCrossListVO o1, AbnormalCrossListVO o2) {
            // 先按路口状态排序
            Integer o1status = o1.getRealtimeStatus();
            Integer o2status = o2.getRealtimeStatus();
            int flag = o2status - o1status;
            if (flag == 0) {
                // 再按拥堵时长排序
                Long o1duration = o1.getDuration();
                Long o2Duration = o2.getDuration();
                long res = o2Duration - o1duration;
                return (int) res;
            }
            return flag;
        }
    }

    private void buildRidData(AbnormalCrossListVO abnormalCrossListVO, String endCrossId,
                              List<Integer> spilloverDirList) {
        List<List<Double>> ridLocationList = new ArrayList<>();
        List<String> ridWktList = new ArrayList<>();
        for (Integer spilloverDir : spilloverDirList) {
            RidInfoEntity ridInfo = ridInfoMapper.selectByEndInDir(endCrossId, spilloverDir);
            if (ridInfo != null) {
                // 关联路口坐标
                String startCrossId = ridInfo.getStartCrossId();
                TBaseCrossInfo crossInfoPO = baseCrossInfoMapper.selectByPrimaryKey(startCrossId);
                if (crossInfoPO != null) {
                    String startLocationStr = crossInfoPO.getLocation();
                    double[] startLonLat = CrossUtil.getLonLat(startLocationStr);
                    List<Double> startLocation = new ArrayList<>();
                    startLocation.add(startLonLat[0]);
                    startLocation.add(startLonLat[1]);
                    ridLocationList.add(startLocation);
                }
                // 关联路口wkt
                String wkt = ridInfo.getWkt();
                ridWktList.add(wkt);
            }
        }
        abnormalCrossListVO.setRidLocationList(ridLocationList);
        abnormalCrossListVO.setRidWktList(ridWktList);
    }

    @Override
    public AbnormalCrossDetailVO abnormalCrossDetail(CrossIdBO crossIdBO) {
        String crossId = crossIdBO.getCrossId();
        AbnormalCrossDetailVO abnormalCrossDetailVO = new AbnormalCrossDetailVO();

        // 构造路口实时数据
        CrossDataRealtimePO crossDataRealtimePO = crossDataRealtimeMapper.selectByCrossId(crossId);
        abnormalCrossDetailVO.setCrossStatus(crossDataRealtimePO.getStatus());
        abnormalCrossDetailVO.setCrossQueueLength(crossDataRealtimePO.getQueueLength());
        abnormalCrossDetailVO.setCrossFlowRate((double)crossDataRealtimePO.getFlow());
        abnormalCrossDetailVO.setCrossStopTimes(crossDataRealtimePO.getStopTimes());
        BigDecimal bigDecimal = BigDecimal.valueOf(crossDataRealtimePO.getCongestionIndex()).setScale(2, RoundingMode.HALF_UP);
        abnormalCrossDetailVO.setCongestionIndex(bigDecimal);
        // 构造转向实时数据
        List<AbnormalCrossDetailVO.TurnDataElement> turnData = buildTurnData(crossId);
        abnormalCrossDetailVO.setTurnData(turnData);
        // 构造方向历史数据
        List<AbnormalCrossDetailVO.DirDataElement> dirData = buildDirData(crossId);
        abnormalCrossDetailVO.setDirData(dirData);

        return abnormalCrossDetailVO;
    }

    @Override
    public List<EventAlarmVO> eventAlarm(Date todayTime) throws ParseException {
        List<EventAlarmPO> eventAlarmPOList = eventAlarmMapper.selectByTodayTime(todayTime);
        List<EventAlarmVO> eventAlarmOutVOList = new ArrayList<>();
        for (EventAlarmPO eventAlarmPO : eventAlarmPOList) {
            EventAlarmVO eventAlarmOutVO = new EventAlarmVO();
            eventAlarmOutVO.setEventDesc(eventAlarmPO.getEventDesc());
            // 事件类型
            Integer typeCode = eventAlarmPO.getType();
            String msgByCode = EventAlarmEnum.getMsgByCode(typeCode);
            eventAlarmOutVO.setType(msgByCode);

            Date endTime = eventAlarmPO.getEndTime();
            if (typeCode == 1 || typeCode == 2) {
                // 发生时间
                Date startTime = eventAlarmPO.getStartTime();
                String startTimeStr = sdf.format(startTime);
                String substring = startTimeStr.substring(11, 16);
                eventAlarmOutVO.setStartTime(substring);
                // 时长
                if (endTime == null) {
                    Date nowTime = DateUtil.date();
                    long between = DateUtil.between(startTime, nowTime, DateUnit.MINUTE);
                    int betweenInt = (int) between;
                    eventAlarmOutVO.setDuration(betweenInt);
                } else {
                    long between = DateUtil.between(startTime, endTime, DateUnit.MINUTE);
                    int betweenInt = (int) between;
                    eventAlarmOutVO.setDuration(betweenInt);
                }
            }
            if (typeCode == 3 || typeCode == 4) {
                // 时间范围
                Date startTime = eventAlarmPO.getStartTime();
                String startTimeStr = sdf.format(startTime);
                String startTimeSubStr = startTimeStr.substring(0, 16); // 截取到分钟
                String endTimeStr = sdf.format(endTime);
                String endTimeSubStr = endTimeStr.substring(0, 16); // 截取到分钟
                eventAlarmOutVO.setTimeSpan(startTimeSubStr + "~" + endTimeSubStr);
            }

            Integer sourceCode = eventAlarmPO.getSource();
            String sourceStr = EventAlarmSourceEnum.getMsgByCode(sourceCode);
            eventAlarmOutVO.setSource(sourceStr);
            eventAlarmOutVO.setCoordinate(eventAlarmPO.getWkt());

            eventAlarmOutVOList.add(eventAlarmOutVO);

        }
        return eventAlarmOutVOList;
    }

    @Override
    public List<GreenWaveCrossMonitorVO> greenwaveCrossMonitor(GreenwaveIdBO greenwaveIdBO) {
        Integer greenwaveId = greenwaveIdBO.getGreenwaveId();
        List<GreenwaveCrossPO> greenwaveCrossPOList = greenwaveCrossMapper.selectByGreenwaveId(greenwaveId);
        List<GreenWaveCrossMonitorVO> res = buildRes(greenwaveCrossPOList);
        return res;
    }

    @Override
    public List<GreenwaveRunMonitorVO> greenwaveRunMonitor(GreenwaveIdBO greenwaveIdBO) {
        List<GreenwaveRunMonitorVO> res = new ArrayList<>();
        Date nowTime = new Date();
        List<GreenwaveHistPOExt> extList = greenwaveHistMapper.selectRunMonitor(nowTime);
        for (GreenwaveHistPOExt greenwaveHistPOExt : extList) {
            GreenwaveRunMonitorVO greenWaveRunMonitorVO = new GreenwaveRunMonitorVO();
            String hour = greenwaveHistPOExt.getHour();
            String hourSub = hour.substring(11, 16); // 截取小时
            greenWaveRunMonitorVO.setHour(hourSub);
            greenWaveRunMonitorVO.setAvgSpeed(greenwaveHistPOExt.getAvgSpeed());
            greenWaveRunMonitorVO.setCongestionIndex(greenwaveHistPOExt.getCongestionIndex());
            res.add(greenWaveRunMonitorVO);
        }
        return res;
    }

    @Override
    public GreenwaveDetailVO greenwaveDetail(GreenwaveDetailBO greenwaveDetailBO) {
        String greenwaveName = greenwaveDetailBO.getGreenwaveName();
        String startDate = greenwaveDetailBO.getStartDate();
        String endDate = greenwaveDetailBO.getEndDate();

        LineSchemeDTO lineSchemeDTO = MainlineEvaluateServiceImpl.lineSchemeBuffer.get(greenwaveName);
        Integer greenwaveId = lineSchemeDTO.getGreenwaveId();

        GreenwaveDetailVO res = new GreenwaveDetailVO();
        GreenwaveRealtimePO greenwaveRealtimePO = greenwaveRealtimeMapper.selectById(greenwaveId);
        GreenwaveInfoPO greenwaveInfoPO = greenwaveInfoMapper.selectById(greenwaveId);
        res.setName(greenwaveInfoPO.getName());
        res.setStatus(greenwaveRealtimePO.getStatus());

        // 绿波带宽和设计速度
        List<GreenwaveCrossPOExt> greenwaveCrossPOExtList = greenwaveCrossMapper.selectAllWidthAndSpeed();
        String[] split = greenwaveName.split(" "); // 旅游路 东向西 转山西路至霞景路路段:正向绿波 工作日 19:00-20:00
        String baseGreenwaveName = split[0] + " " + split[1] + " " + split[2]; // 旅游路 东向西 转山西路至霞景路路段:正向绿波
        String planName = split[3]; // 工作日
        String timeSpan = split[4]; // 19:00-20:00
        String[] split1 = timeSpan.split("-");
        String startTime = split1[0];

        Optional<GreenwaveCrossPOExt> result = greenwaveCrossPOExtList.stream()
                .filter(item -> baseGreenwaveName.equals(item.getGreenwaveName()) &&
                        planName.equals(item.getPlanName()) &&
                        startTime.equals(item.getStartTime()))
                .findFirst();
        GreenwaveCrossPOExt foundItem = result.get();

        double width = foundItem.getWidth();
        double designSpeed = foundItem.getDesignSpeed() / 3.6; // 换算成米/秒
        int widthSeconds = (int)(width / designSpeed);
        Integer dir = greenwaveInfoPO.getDir();
        if (dir == 0) { // 正向
            res.setForwardWidth(widthSeconds);
        } else if (dir == 1) { // 反向
            res.setReverseWidth(widthSeconds);
        } else if (dir == 2) { // 双向
            res.setForwardWidth(widthSeconds);
            res.setReverseWidth(widthSeconds);
        }

        double uncoordinatePhaseQueue = greenwaveRealtimePO.getUncoordinatePhaseQueue();
        res.setUncoordinatePhaseQueue((int)uncoordinatePhaseQueue);
        double noparkPassRate = greenwaveRealtimePO.getNoparkPassRate();
        res.setNoparkPassRate((int) (noparkPassRate * 100));
        res.setTrvalTime(greenwaveRealtimePO.getTrvalTime());

        // 策略列表
        List<String> strategyList = strategyMapper.selectGreenwaveAllNames();
        res.setStrategyList(strategyList);
        List<StrategyPO> strategyPOList = strategyMapper.selectByGreenwaveId(greenwaveId);
        List<String> collect = strategyPOList.stream()
                .map(StrategyPO::getStrategyName)
                .collect(Collectors.toList());
        res.setStrategySelected(collect);
        // 绿波路口信息
        List<GreenwaveDetailVO.GreenwaveCross> greenwaveCrossList = buildGreenwaveCrossList(
                greenwaveId, greenwaveName, startDate, endDate);
        res.setGreenwaveCrossList(greenwaveCrossList);

        // 周期时长
        GreenwaveDetailVO.GreenwaveCross greenwaveCross = greenwaveCrossList.get(0);
        List<GreenwaveDetailVO.GreenwavePhase> phaseList = greenwaveCross.getGreenwavePhaseList();
        int cycle = phaseList.stream()
                .mapToInt(GreenwaveDetailVO.GreenwavePhase::getSeconds)
                .sum();
        res.setCycle(cycle);

        // 指针所在秒数
        res.setPointerSecond(calcPointerSecond(startTime, cycle));

        // 绿波带
        if (dir == 0) { // 正向
            List<GreenwaveDetailVO.GreenwavePoint> greenwavePoints =
                    buildForwardGreenwaveList(greenwaveCrossList, greenwaveInfoPO);
            addStartOffset(greenwavePoints, greenwaveCrossList);
            res.setForwardGreenwaveList(greenwavePoints);
        } else if (dir == 1) { // 反向
            List<GreenwaveDetailVO.GreenwavePoint> greenwavePoints =
                    buildReverseGreenwaveList(greenwaveCrossList, greenwaveInfoPO);
            addStartOffset(greenwavePoints, greenwaveCrossList);
            res.setReverseGreenwaveList(greenwavePoints);
        } else if (dir == 2) { // 双向
            List<GreenwaveDetailVO.GreenwavePoint> greenwavePoints =
                    buildForwardGreenwaveList(greenwaveCrossList, greenwaveInfoPO);
            addStartOffset(greenwavePoints, greenwaveCrossList);
            res.setForwardGreenwaveList(greenwavePoints);
            List<GreenwaveDetailVO.GreenwavePoint> greenwavePoints1 =
                    buildReverseGreenwaveList(greenwaveCrossList, greenwaveInfoPO);
            addStartOffset(greenwavePoints1, greenwaveCrossList);
            res.setReverseGreenwaveList(greenwavePoints1);
        }

        return res;
    }

    private void addStartOffset(List<GreenwaveDetailVO.GreenwavePoint> greenwavePoints,
                                List<GreenwaveDetailVO.GreenwaveCross> greenwaveCrossList) {
        for (GreenwaveDetailVO.GreenwavePoint greenwavePoint : greenwavePoints) {
            String crossId = greenwavePoint.getCrossId();
            for (GreenwaveDetailVO.GreenwaveCross greenwaveCross : greenwaveCrossList) {
                String crossId1 = greenwaveCross.getCrossId();
                if (Objects.equals(crossId1, crossId)) {
                    Integer coordOffset = greenwavePoint.getCoordOffset();
                    Integer seconds = calcTimeFromStart(greenwaveCross);
                    greenwavePoint.setOffsetToStart(coordOffset + seconds);
                }
            }
        }
    }

    private Integer calcTimeFromStart(GreenwaveDetailVO.GreenwaveCross greenwaveCross) {
        int res = 0;
        List<GreenwaveDetailVO.GreenwavePhase> phaseList = greenwaveCross.getGreenwavePhaseList();
        for (GreenwaveDetailVO.GreenwavePhase greenwavePhase : phaseList) {
            Integer seconds = greenwavePhase.getSeconds();
            Integer isCoordinate = greenwavePhase.getIsCoordinate();
            if (1 != isCoordinate) {
                res += seconds;
            } else {
                break;
            }
        }
        return res;
    }

    private Integer calcPointerSecond(String startTime, int cycle) {
        // 计算当前小时分钟数与录波开始时间相差的秒数
        int currentSecondDiff = calcCurrentSecondDiff(startTime);
        // 计算相差秒数除以相位周期的余数
        int res = currentSecondDiff % cycle;
        if (res == 0) {
            res = cycle;
        }
        return res;
    }

    private int calcCurrentSecondDiff(String startTime) {
        LocalTime now = LocalTime.now();
        LocalTime startTimeObj = LocalTime.parse(startTime);
        Duration duration = Duration.between(startTimeObj, now);
        long seconds = duration.getSeconds();
        return (int)seconds;
    }

    private List<GreenwaveDetailVO.GreenwavePoint> buildReverseGreenwaveList(
            List<GreenwaveDetailVO.GreenwaveCross> greenwaveCrossList, GreenwaveInfoPO greenwaveInfoPO) {
        List<GreenwaveDetailVO.GreenwavePoint> res = new ArrayList<>();
        // 找关键路口
        GreenwaveDetailVO.GreenwaveCross keyCross = null;
        Integer keyCrossBeginSeconds = null;
        for (GreenwaveDetailVO.GreenwaveCross greenwaveCross : greenwaveCrossList) {
            if (1 == greenwaveCross.getIsKeyRoute()) {
                keyCross = greenwaveCross;
                GreenwaveDetailVO.GreenwavePoint greenwavePoint = buildGreenwavePoint(greenwaveCross);
                Integer coordSeconds = greenwavePoint.getCoordSeconds();
                greenwavePoint.setCoordOffset(coordSeconds);
                res.add(greenwavePoint);
                keyCrossBeginSeconds = calcCoordBeginSeconds(greenwaveCross) + coordSeconds;
            }
        }
        Integer sort = keyCross.getSort();
        // 从关键路口向上
        int upSize = sort - 2;
        Integer preCrossBeginSeconds = keyCrossBeginSeconds;
        for (int i = upSize; i >= 0; i--) {
            GreenwaveDetailVO.GreenwaveCross greenwaveCross = greenwaveCrossList.get(i);
            GreenwaveDetailVO.GreenwavePoint greenwavePoint = buildGreenwavePoint(greenwaveCross);
            // 计算行程时间
            int distanceToNextCross = greenwaveCross.getDistanceToNextCross();
            double designSpeed = greenwaveInfoPO.getDesignSpeed() / 3.6; // 换算成米/秒
            int travalTime = (int)(distanceToNextCross / designSpeed);
            // 计算协调相位绿灯开始的绝对时间
            int absoluteSecondsBegin = calcCoordBeginSeconds(greenwaveCross);
            // 计算绿波点位的绝对时间
            int absoluteSecondsPoint = preCrossBeginSeconds + travalTime;
            greenwavePoint.setCoordOffset(absoluteSecondsPoint - absoluteSecondsBegin);

            res.add(greenwavePoint);
            preCrossBeginSeconds = absoluteSecondsPoint;
        }
        // 从关键路口向下
        preCrossBeginSeconds = keyCrossBeginSeconds;
        for (int i = sort; i < greenwaveCrossList.size(); i++) {
            GreenwaveDetailVO.GreenwaveCross greenwaveCross = greenwaveCrossList.get(i);
            GreenwaveDetailVO.GreenwaveCross greenwaveCrossAbove = greenwaveCrossList.get(i - 1);
            GreenwaveDetailVO.GreenwavePoint greenwavePoint = buildGreenwavePoint(greenwaveCross);
            // 计算行程时间
            int distanceToNextCross = greenwaveCrossAbove.getDistanceToNextCross();
            double designSpeed = greenwaveInfoPO.getDesignSpeed() / 3.6; // 换算成米/秒
            int travalTime = (int)(distanceToNextCross / designSpeed);
            // 计算协调相位绿灯开始的绝对时间
            int absoluteSecondsBegin = calcCoordBeginSeconds(greenwaveCross);
            // 计算绿波点位的绝对时间
            int absoluteSecondsPoint = preCrossBeginSeconds - travalTime;
            greenwavePoint.setCoordOffset(absoluteSecondsPoint - absoluteSecondsBegin);

            res.add(greenwavePoint);
            preCrossBeginSeconds = absoluteSecondsPoint;
        }
        List<GreenwaveDetailVO.GreenwavePoint> sorted = res.stream()
                .sorted(Comparator.comparing(GreenwaveDetailVO.GreenwavePoint::getSort))
                .collect(Collectors.toList());
        return sorted;
    }

    private List<GreenwaveDetailVO.GreenwavePoint> buildForwardGreenwaveList(
            List<GreenwaveDetailVO.GreenwaveCross> greenwaveCrossList, GreenwaveInfoPO greenwaveInfoPO) {
        List<GreenwaveDetailVO.GreenwavePoint> res = new ArrayList<>();
        // 找关键路口
        GreenwaveDetailVO.GreenwaveCross keyCross = null;
        Integer keyCrossBeginSeconds = null;
        for (GreenwaveDetailVO.GreenwaveCross greenwaveCross : greenwaveCrossList) {
            if (1 == greenwaveCross.getIsKeyRoute()) {
                keyCross = greenwaveCross;
                GreenwaveDetailVO.GreenwavePoint greenwavePoint = buildGreenwavePoint(greenwaveCross);
                greenwavePoint.setCoordOffset(0);
                res.add(greenwavePoint);
                keyCrossBeginSeconds = calcCoordBeginSeconds(greenwaveCross);
            }
        }
        Integer sort = keyCross.getSort();
        // 从关键路口向上
        int upSize = sort - 2;
        Integer preCrossBeginSeconds = keyCrossBeginSeconds;
        for (int i = upSize; i >= 0; i--) {
            GreenwaveDetailVO.GreenwaveCross greenwaveCross = greenwaveCrossList.get(i);
            GreenwaveDetailVO.GreenwavePoint greenwavePoint = buildGreenwavePoint(greenwaveCross);
            // 计算行程时间
            int distanceToNextCross = greenwaveCross.getDistanceToNextCross();
            double designSpeed = greenwaveInfoPO.getDesignSpeed() / 3.6; // 换算成米/秒
            int travalTime = (int)(distanceToNextCross / designSpeed);
            // 计算协调相位绿灯开始的绝对时间
            int absoluteSecondsBegin = calcCoordBeginSeconds(greenwaveCross);
            // 计算绿波点位的绝对时间
            int absoluteSecondsPoint = preCrossBeginSeconds - travalTime;
            greenwavePoint.setCoordOffset(absoluteSecondsPoint - absoluteSecondsBegin);

            res.add(greenwavePoint);
            preCrossBeginSeconds = absoluteSecondsPoint;
        }
        // 从关键路口向下
        preCrossBeginSeconds = keyCrossBeginSeconds;
        for (int i = sort; i < greenwaveCrossList.size(); i++) {
            GreenwaveDetailVO.GreenwaveCross greenwaveCross = greenwaveCrossList.get(i);
            GreenwaveDetailVO.GreenwaveCross greenwaveCrossAbove = greenwaveCrossList.get(i - 1);
            GreenwaveDetailVO.GreenwavePoint greenwavePoint = buildGreenwavePoint(greenwaveCross);
            // 计算行程时间
            int distanceToNextCross = greenwaveCrossAbove.getDistanceToNextCross();
            double designSpeed = greenwaveInfoPO.getDesignSpeed() / 3.6; // 换算成米/秒
            int travalTime = (int)(distanceToNextCross / designSpeed);
            // 计算协调相位绿灯开始的绝对时间
            int absoluteSecondsBegin = calcCoordBeginSeconds(greenwaveCross);
            // 计算绿波点位的绝对时间
            int absoluteSecondsPoint = preCrossBeginSeconds + travalTime;
            greenwavePoint.setCoordOffset(absoluteSecondsPoint - absoluteSecondsBegin);

            res.add(greenwavePoint);
            preCrossBeginSeconds = absoluteSecondsPoint;
        }
        List<GreenwaveDetailVO.GreenwavePoint> sorted = res.stream()
                .sorted(Comparator.comparing(GreenwaveDetailVO.GreenwavePoint::getSort))
                .collect(Collectors.toList());
        return sorted;
    }

    private int calcCoordBeginSeconds(GreenwaveDetailVO.GreenwaveCross greenwaveCross) {
        int res = 0;
        List<GreenwaveDetailVO.GreenwavePhase> greenwavePhaseList = greenwaveCross.getGreenwavePhaseList();
        for (GreenwaveDetailVO.GreenwavePhase greenwavePhase : greenwavePhaseList) {
            int seconds = greenwavePhase.getSeconds();
            int isCoordinate = greenwavePhase.getIsCoordinate();
            if (1 != isCoordinate) {
                res += seconds;
            } else {
                break;
            }
        }
        return res;
    }

    private GreenwaveDetailVO.GreenwavePoint buildGreenwavePoint(GreenwaveDetailVO.GreenwaveCross greenwaveCross) {
        GreenwaveDetailVO.GreenwavePoint greenwavePoint = new GreenwaveDetailVO.GreenwavePoint();
        greenwavePoint.setSort(greenwaveCross.getSort());
        greenwavePoint.setCrossId(greenwaveCross.getCrossId());
        greenwavePoint.setCrossName(greenwaveCross.getCrossName());
        List<GreenwaveDetailVO.GreenwavePhase> greenwavePhaseList = greenwaveCross.getGreenwavePhaseList();
        for (GreenwaveDetailVO.GreenwavePhase greenwavePhase : greenwavePhaseList) {
            if (1 == greenwavePhase.getIsCoordinate()) {
                greenwavePoint.setCoordSeconds(greenwavePhase.getSeconds());
            }
        }
        return greenwavePoint;
    }

    private List<GreenwaveDetailVO.GreenwaveCross> buildGreenwaveCrossList(
            Integer greenwaveId, String greenwaveName, String startDate, String endDate) {

        String[] split = greenwaveName.split(" "); // 旅游路 东向西 转山西路至霞景路路段:正向绿波 工作日 19:00-20:00
        String s = split[4]; // 19:00-20:00
        String[] split1 = s.split("-");
        String startTime = split1[0];
        String endTime = split1[1];
        String planName = split[3];

        List<GreenwaveDetailVO.GreenwaveCross> res = new ArrayList<>();
        List<GreenwaveCrossPO> greenwaveCrossList = greenwaveCrossMapper.selectByGreenwaveId(greenwaveId);

        // 获取绿波的路口ID列表
        List<String> crossList = greenwaveCrossList.stream()
                .map(GreenwaveCrossPO::getCrossId)
                .distinct()
                .collect(Collectors.toList());

        // 查询路口时段表基础数据
        List<CrossSectionPOExt> crossSectionPoExtList =
                baseCrossSectionMapper.selectByCrossIdsAndStartTime(startTime, crossList);

        // 根据方案名和开始时间过滤
        List<Integer> sectionIdList = crossSectionPoExtList.stream()
                .filter(poExt -> planName.equals(poExt.getPlanName()) && startTime.equals(poExt.getStartTime()))
                .map(CrossSectionPOExt::getId)
                .collect(Collectors.toList());

        // 根据时段ID过滤数据
        List<GreenwaveCrossPO> filteredList = greenwaveCrossList.stream()
                .filter(greenwaveCross -> sectionIdList.contains(greenwaveCross.getSectionId()))
                .collect(Collectors.toList());
        filteredList.sort(Comparator.comparing(GreenwaveCrossPO::getSort));

        // 相对相位差列表
        List<Integer> relativeOffsets = filteredList.stream()
                .map(GreenwaveCrossPO::getOffset)
                .collect(Collectors.toList());
        // 绝对相位差列表
        List<Integer> absoluteOffsets = buildAbsoluteOffsets(relativeOffsets);

        int size = filteredList.size();
        for (int i = 0; i < size; i++) {
            GreenwaveCrossPO greenwaveCrossPO = filteredList.get(i);
            GreenwaveDetailVO.GreenwaveCross greenwaveCross = new GreenwaveDetailVO.GreenwaveCross();
            String crossId = greenwaveCrossPO.getCrossId();
            greenwaveCross.setCrossId(crossId);
            BaseCrossInfoPO baseCrossInfoPO = baseCrossInfoMapper.selectById(crossId);
            if (baseCrossInfoPO == null) {
                throw new RuntimeException("无此路口基础信息");
            }
            greenwaveCross.setCrossName(baseCrossInfoPO.getName());
            greenwaveCross.setIsKeyRoute(greenwaveCrossPO.getIsKeyRoute());
            greenwaveCross.setSort(greenwaveCrossPO.getSort());
            // 到下一个路口距离和速度
            Integer distanceToNextCross = null;
            Double speed = null;
            if (i < size - 1) { // 不是最后一个路口
                double nextCrossLen = greenwaveCrossPO.getNextCrossLen();
                distanceToNextCross = (int)nextCrossLen;

                GreenwaveCrossPO nextCrossPO = filteredList.get(i + 1);
                String nextCrossId = nextCrossPO.getCrossId();
                speed = calcSpeed(greenwaveName, nextCrossId, startTime, endTime, startDate, endDate);
            }
            greenwaveCross.setDistanceToNextCross(distanceToNextCross);
            greenwaveCross.setSpeed(speed);
            // 相位差
            greenwaveCross.setOffset(greenwaveCrossPO.getOffset());

            // 获取当前方案ID
            BaseCrossPlanPO baseCrossPlanPO = baseCrossPlanMapper.selectByCrossIdAndName(crossId, planName);
            Integer planId = baseCrossPlanPO.getId();
            CrossSectionPO crossSectionPO = baseCrossSectionMapper.
                    selectbyStartTimeCrossIdPlanId(startTime, crossId, planId);
            if (crossSectionPO == null) {
                throw new RuntimeException("无当前时段方案信息");
            }
            Integer currentSchemeId = crossSectionPO.getSchemeId();
            BaseCrossSchemePO baseCrossSchemePO = baseCrossSchemeMapper.selectById(currentSchemeId);

            // 构造相位时差图
            Integer absoluteOffset = absoluteOffsets.get(i);
            Integer crossSort = greenwaveCrossPO.getSort();
            List<GreenwaveDetailVO.GreenwavePhase> greenwavePhaseList =
                    getGreenwavePhaseList(baseCrossSchemePO, absoluteOffset, crossSort);
            greenwaveCross.setGreenwavePhaseList(greenwavePhaseList);

            res.add(greenwaveCross);
        }
        return res;
    }

    private List<Integer> buildAbsoluteOffsets(List<Integer> relativeOffsets) {
        List<Integer> res = new ArrayList<>();
        for (int i = 0; i < relativeOffsets.size(); i++) {
            Integer subSum = 0;
            for (int j = 0; j <= i; j++) {
                subSum += relativeOffsets.get(j);
            }
            res.add(subSum);
        }
        return res;
    }

    private List<GreenwaveDetailVO.GreenwavePhase> getGreenwavePhaseList(
            BaseCrossSchemePO baseCrossSchemePO, Integer absoluteOffset, Integer crossSort) {

        List<GreenwaveDetailVO.GreenwavePhase> res = new ArrayList<>();
        Integer schemeId = baseCrossSchemePO.getId();
        String crossId = baseCrossSchemePO.getCrossId();
        Integer coordPhaseId = baseCrossSchemePO.getCoordPhase();
        // 获取方案相位并按相位序排序
        List<CrossPhasePO> crossPhasePOList = baseCrossPhaseMapper.selectByCrossIdAndSchemeId(crossId, schemeId);
        if (1 == crossSort) { // 第一个路口没有相位差
            int size = crossPhasePOList.size();
            for (int i = 0; i < size; i++) {
                CrossPhasePO crossPhasePO = crossPhasePOList.get(i);
                GreenwaveDetailVO.GreenwavePhase greenwavePhase = new GreenwaveDetailVO.GreenwavePhase();

                greenwavePhase.setSort(i + 1);
                greenwavePhase.setPhaseNo(crossPhasePO.getPhaseNo());
                greenwavePhase.setSeconds(crossPhasePO.getPhaseTime());
                Integer phaseId = crossPhasePO.getId();
                greenwavePhase.setIsCoordinate(ObjectUtil.equals(phaseId, coordPhaseId) ? 1 : 0);

                res.add(greenwavePhase);
            }
            return res;
        } else {
            // 其余路口，首先根据绝对相位差构造第一个相位，与最后一个相位首位相接
            GreenwaveDetailVO.GreenwavePhase firstGreenwavePhase = new GreenwaveDetailVO.GreenwavePhase();
            int size = crossPhasePOList.size();
            CrossPhasePO endCrossPhasePO = crossPhasePOList.get(size - 1);
            firstGreenwavePhase.setSort(1);
            firstGreenwavePhase.setPhaseNo(endCrossPhasePO.getPhaseNo());
            firstGreenwavePhase.setSeconds(absoluteOffset);
            Integer firstPhaseId = endCrossPhasePO.getId();
            firstGreenwavePhase.setIsCoordinate(ObjectUtil.equals(firstPhaseId, coordPhaseId) ? 1 : 0);
            res.add(firstGreenwavePhase);

            for (int i = 0; i < size; i++) {
                CrossPhasePO crossPhasePO = crossPhasePOList.get(i);
                GreenwaveDetailVO.GreenwavePhase greenwavePhase = new GreenwaveDetailVO.GreenwavePhase();

                greenwavePhase.setSort(i + 2);
                greenwavePhase.setPhaseNo(crossPhasePO.getPhaseNo());
                if (i == size - 1){
                    greenwavePhase.setSeconds(crossPhasePO.getPhaseTime() - absoluteOffset);
                } else {
                    greenwavePhase.setSeconds(crossPhasePO.getPhaseTime());
                }
                Integer phaseId = crossPhasePO.getId();
                greenwavePhase.setIsCoordinate(ObjectUtil.equals(phaseId, coordPhaseId) ? 1 : 0);

                res.add(greenwavePhase);
            }
            return res;
        }
    }

    private int convertWeekDay(int i) {
        if (1 == i) {
            return 7;
        }
        return i - 1;
    }

    private Double calcSpeed(String greenwaveName, String crossId,
                             String startTime, String endTime, String startDate, String endDate) {
        // 确定协调方向
        String[] split = greenwaveName.split(" ");
        String dirStr = split[1];
        String[] split1 = dirStr.split("向");
        String lineName = split[0] + " " + split[1] + " " + split[2];
        GreenwaveInfoPO greenwaveInfoPO = greenwaveInfoMapper.selectByName(lineName);
        Integer dir = greenwaveInfoPO.getDir();

        List<Integer> coordDirCodeList = new ArrayList<>();
        if (dir == 0) { // 正向
            String dirStr1 = split1[0];
            Integer dirCode = BaseEnum.SignalDirectionEnum.getCodeByName(dirStr1);
            dirCode = convertDirCode(dirStr1, crossId, dirCode);
            coordDirCodeList.add(dirCode);
        } else if (dir == 1) { // 反向
            String dirStr1 = split1[1];
            Integer dirCode = BaseEnum.SignalDirectionEnum.getCodeByName(dirStr1);
            dirCode = convertDirCode(dirStr1, crossId, dirCode);
            coordDirCodeList.add(dirCode);
        } else { // 双向
            String dirStr1 = split1[0];
            Integer dirCode1 = BaseEnum.SignalDirectionEnum.getCodeByName(dirStr1);
            dirCode1 = convertDirCode(dirStr1, crossId, dirCode1);
            coordDirCodeList.add(dirCode1);
            String dirStr2 = split1[1];
            Integer dirCode2 = BaseEnum.SignalDirectionEnum.getCodeByName(dirStr2);
            dirCode2 = convertDirCode(dirStr2, crossId, dirCode2);
            coordDirCodeList.add(dirCode2);
        }

        if (ObjectUtil.isEmpty(startDate)) { // 查询实时数据
            CrossDirDataRealtimePO crossDirDataRealtimePO =
                    crossDirDataRealtimeMapper.selectByCrossIdAndDirs(crossId, coordDirCodeList);
            return crossDirDataRealtimePO.getSpeed();
        } else { // 查询历史数据
            List<CrossDirDataHistPO> crossDirDataHistPOList = crossDirDataHistMapper.selectByCrossDirsAndTimeSection(
                    crossId, coordDirCodeList, startDate, endDate, startTime, endTime);

            double v = crossDirDataHistPOList.stream()
                    .mapToDouble(CrossDirDataHistPO::getSpeed)
                    .average().orElse(0.0);
            BigDecimal formattedAverage = new BigDecimal(v).setScale(2, RoundingMode.HALF_UP);
            return formattedAverage.doubleValue();
        }
    }

    @Override
    public List<GreenwaveCrossMetricsVO> greenwaveCrossMetrics(GreenwaveIdAndTimeStampBO greenwaveIdAndTimeStampBO) {
        List<GreenwaveCrossMetricsVO> res = new ArrayList<>();
        Long timeStamp = greenwaveIdAndTimeStampBO.getTimeStamp();

        Integer greenwaveId = greenwaveIdAndTimeStampBO.getGreenwaveId();
        GreenwaveInfoPO greenwaveInfoPO = greenwaveInfoMapper.selectById(greenwaveId);
        String greenwaveName = greenwaveInfoPO.getName(); // 旅游路 东向西 转山西路至霞景路路段:正向绿波
        String[] split = greenwaveName.split(" ");
        String[] split1 = split[1].split("向"); // 东向西
        String dirName1 = split1[0];
        String dirName2 = split1[1];

        List<GreenwaveCrossPO> greenwaveCrossList = greenwaveCrossMapper.selectByGreenwaveId(greenwaveId);
        // 根据路口ID去重，并按路口顺序号排序
        Map<String, GreenwaveCrossPO> uniqueMap = greenwaveCrossList.stream()
                .collect(Collectors.toMap(
                        GreenwaveCrossPO::getCrossId,    // 使用crossId作为键
                        Function.identity(),              // 使用对象本身作为值
                        (existing, replacement) -> existing));  // 如果有重复的key, 保留第一个

        // 从uniqueMap中获取去重后的记录列表
        List<GreenwaveCrossPO> uniqueList = new ArrayList<>(uniqueMap.values());

        for (GreenwaveCrossPO greenwaveCrossPO : uniqueList) {
            GreenwaveCrossMetricsVO greenwaveCrossMetricsVO = new GreenwaveCrossMetricsVO();
            String crossId = greenwaveCrossPO.getCrossId();
            greenwaveCrossMetricsVO.setCrossId(crossId);
            BaseCrossInfoPO baseCrossInfoPO = baseCrossInfoMapper.selectById(crossId);
            if (baseCrossInfoPO != null) {
                greenwaveCrossMetricsVO.setCrossName(baseCrossInfoPO.getName());
            }
            List<Integer> coordDirList = new ArrayList<Integer>();
            int direction = greenwaveInfoPO.getDir();
            if (direction == 0) { // 正向
                Integer dirCode = BaseEnum.SignalDirectionEnum.getCodeByName(dirName1);
                dirCode = convertDirCode(dirName1, crossId, dirCode);
                coordDirList.add(dirCode);
            } else if (direction == 2) { // 反向
                Integer dirCode = BaseEnum.SignalDirectionEnum.getCodeByName(dirName2);
                dirCode = convertDirCode(dirName2, crossId, dirCode);
                coordDirList.add(dirCode);
            } else { // 双向
                Integer dirCode1 = BaseEnum.SignalDirectionEnum.getCodeByName(dirName1);
                dirCode1 = convertDirCode(dirName1, crossId, dirCode1);
                coordDirList.add(dirCode1);
                Integer dirCode2 = BaseEnum.SignalDirectionEnum.getCodeByName(dirName2);
                dirCode2 = convertDirCode(dirName1, crossId, dirCode2);
                coordDirList.add(dirCode2);
            }

            int currentSeconds = (int) (timeStamp / 1000);
            int preSeconds = (int) (currentSeconds - 60 * 5);
            List<CrossDirDataHistPO> crossDirDataHistPOList =
                    crossDirDataHistMapper.selectByCrossIdAndStartEnd(crossId, preSeconds, currentSeconds);

            greenwaveCrossMetricsVO.setNoparkPassRate(calcNoparkPassRate(crossDirDataHistPOList, coordDirList));
            greenwaveCrossMetricsVO.setUncoordinatePhaseQueue(
                    calcUncoordinatePhaseQueue(crossDirDataHistPOList, coordDirList));
            Double nextCrossLen = greenwaveCrossPO.getNextCrossLen();
            greenwaveCrossMetricsVO.setTrvalTime(calcTravelTime(crossDirDataHistPOList, coordDirList, nextCrossLen));

            res.add(greenwaveCrossMetricsVO);
        }
        return res;
    }

    private Integer calcTravelTime(List<CrossDirDataHistPO> crossDirDataHistPOList, List<Integer> coordDirList,
                                   Double nextCrossLen) {
        double average = crossDirDataHistPOList.stream()
                .filter(po -> po.getInOutType() == 1 && coordDirList.contains(po.getDirType()))
                .mapToDouble(CrossDirDataHistPO::getSpeed)
                .average()
                .orElse(0.0);
        if (average != 0.0) {
            return (int) (nextCrossLen / average);
        } else {
            return 0;
        }
    }

    private Integer calcUncoordinatePhaseQueue(
            List<CrossDirDataHistPO> crossDirDataHistPOList, List<Integer> coordDirList) {
        double average = crossDirDataHistPOList.stream()
                .filter(po -> po.getInOutType() == 1 && !coordDirList.contains(po.getDirType()))
                .mapToDouble(CrossDirDataHistPO::getQueueLength)
                .average()
                .orElse(0.0);

        return (int) average;
    }

    private Integer calcNoparkPassRate(List<CrossDirDataHistPO> crossDirDataHistPOList, List<Integer> coordDirList) {
        double average = crossDirDataHistPOList.stream()
                .filter(po -> po.getInOutType() == 1 && coordDirList.contains(po.getDirType()))
                .mapToDouble(CrossDirDataHistPO::getNoStopRate)
                .average()
                .orElse(0.0);

        return (int) (average * 100);
    }

    private Integer convertDirCode(String dirStr, String crossId, Integer dirCode) {
        if (ObjectUtil.isNotEmpty(directionMappingsConfig)) {
            List<String> dirMapList = directionMappingsConfig.getMappings().get(crossId); // 如 ["东:2","西:6"]
            if (CollectionUtil.isNotEmpty(dirMapList)) {
                for (String dirMapStr : dirMapList) {
                    String[] split = dirMapStr.split(":");
                    if (dirStr.equals(split[0])) {
                        dirCode = Integer.parseInt(split[1]);
                    }
                }
            }
        }
        return dirCode;
    }

    private List<CrossTurnDataHistPO> getPhaseQueueList(
            List<CrossTurnDataHistPO> listForNoPark, List<CrossTurnDataHistPO> allPO) {
        // 去除协调相位，用于计算非协调相位二次排队
        Iterator<CrossTurnDataHistPO> iterator = allPO.iterator();
        while (iterator.hasNext()) {
            CrossTurnDataHistPO next = iterator.next();
            for (CrossTurnDataHistPO crossTurnDataHistPO : listForNoPark) {
                if (Objects.equals(next.getTurnType(), crossTurnDataHistPO.getTurnType())
                        && Objects.equals(next.getInDir(), crossTurnDataHistPO.getInDir())) {
                    iterator.remove();
                    break;
                }
            }
        }
        return allPO;
    }

    @Override
    @Transactional
    public void saveGreenwaveStrategy(SaveGreenwaveStrategyBO saveGreenwaveStrategyBO) {
        Integer greenwaveId = saveGreenwaveStrategyBO.getGreenwaveId();
        List<String> strategyNames = saveGreenwaveStrategyBO.getStrategyNames();
        GreenwaveInfoPO greenwaveInfoPO = greenwaveInfoMapper.selectById(greenwaveId);
        Integer dir = greenwaveInfoPO.getDir();
        // 删除旧关联
        greenwaveSceneMapper.deleteByGreenwaveId(greenwaveId);
        for (String strategyName : strategyNames) {
            // 获取策略ID
            StrategyPO strategyPO = strategyMapper.selectByName(strategyName);
            Integer strategyId = strategyPO.getId();
            // 根据策略ID查询场景IDs
            List<Integer> sceneIds = sceneStrategyMapper.selectSceneIdsByStrategyId(strategyId);
            // 插入新关联
            for (Integer sceneId : sceneIds) {
                // 查询场景代码
                ScenePO scenePO = sceneMapper.selectById(sceneId);
                Integer sceneNum = scenePO.getSceneNum();
                Integer sceneCodeByDir = SceneDirEnum.getSceneNumByDir(dir);
                if (Objects.equals(sceneNum, sceneCodeByDir)) {
                    GreenwaveScenePO greenwaveScenePO = new GreenwaveScenePO();
                    greenwaveScenePO.setGreenwaveId(greenwaveId);
                    greenwaveScenePO.setSceneId(sceneId);
                    greenwaveSceneMapper.insertOne(greenwaveScenePO);
                    break;
                }
            }
        }
    }

    @Override
    public GreenwaveDetailVO currentGreenwaveDetail(GreenwaveIdBO bo) throws Exception {

        Integer greenwaveId = bo.getGreenwaveId();
        // 获取绿波名称，调用评价接口
        GreenwaveInfoPO greenwaveInfoPO = greenwaveInfoMapper.selectById(greenwaveId);
        String greenwaveName = greenwaveInfoPO.getName();
        String[] split = greenwaveName.split(":");
        String lineName = split[0];

        MainlineSchemeAnalysisBO mainlineSchemeAnalysisBO = new MainlineSchemeAnalysisBO();
        mainlineSchemeAnalysisBO.setName(lineName);

        // 获取当前日期和时间的 Calendar 对象
        Calendar calendar = Calendar.getInstance();
        // 设置时、分、秒和毫秒为 0，以获取当天的 00:00:00
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        Date startOfDay = calendar.getTime();

        // 设置时、分、秒和毫秒为 23:59:59.999，以获取当天的 23:59:59
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 999);
        Date endOfDay = calendar.getTime();

        mainlineSchemeAnalysisBO.setStartTime(startOfDay);
        mainlineSchemeAnalysisBO.setEndTime(endOfDay);
        MainlineSchemeAnalysisVO mainlineSchemeAnalysisVO = mainlineEvaluateServiceImpl
                .mainlineSchemeAnalysis(mainlineSchemeAnalysisBO);

        // 获取当前时段的干线方案
        List<MainlineSchemeAnalysisVO.GreenwaveData> greenwaveData = mainlineSchemeAnalysisVO.getGreenwaveData();
        MainlineSchemeAnalysisVO.GreenwaveData matchingData = findMatchingData(greenwaveData);
        if (ObjectUtil.isEmpty(matchingData)) {
            throw new RuntimeException("无当天绿波历史数据");
        }
        String lineSchemeName = matchingData.getName();
        GreenwaveDetailBO greenwaveDetailBO = new GreenwaveDetailBO();
        greenwaveDetailBO.setGreenwaveName(lineSchemeName);
        GreenwaveDetailVO greenwaveDetailVO = greenwaveDetail(greenwaveDetailBO);

        return greenwaveDetailVO;
    }

    @Override
    public CountRealTimeVO countRealTime(String crossId) throws Exception {
        CountRealTimeVO countRealTimeVO = new CountRealTimeVO();

        ZoneId zoneId = ZoneId.systemDefault();
        ZonedDateTime startOfDay = LocalDateTime.now().toLocalDate().atStartOfDay(zoneId);
        ZonedDateTime now = ZonedDateTime.now(zoneId);
        int startTimeStamp = (int) startOfDay.toEpochSecond();
        int endTimeStamp = (int) now.toEpochSecond();

        // 统计实时条数
        int counts = laneSnapshotDataQueryService.queryCountsByCrossIdAndTimeSpan(
                crossId, startTimeStamp, endTimeStamp);
        countRealTimeVO.setRealTimeCount(counts);

        // 统计周期条数
        Integer cycleCount = crossLaneDataHistMapper.
                selectCountByCrossIdAndTimeSpan(crossId, startTimeStamp, endTimeStamp);
        countRealTimeVO.setCycleCount(cycleCount);
        return countRealTimeVO;
    }

    @Override
    public TableQueryVO tableQuery(CrossIdAndTimeSpanBO crossIdAndTimeSpanBO) throws Exception {
        String crossId = crossIdAndTimeSpanBO.getCrossId();
        Date startTime = crossIdAndTimeSpanBO.getStartTime();
        Date endTime = crossIdAndTimeSpanBO.getEndTime();
        // 将Date对象转换为10位时间戳
        int startTimeStamp = (int) (startTime.getTime() / 1000);
        int endTimeStamp = (int) (endTime.getTime() / 1000);

        return buildTableQueryVO(crossId, startTimeStamp, endTimeStamp, startTimeStamp);
    }

    private List<TableQueryVO.RealTimeDataElement> buildRealTimeList(List<CrossLaneSnapshotDataDTO> dtoList) {
        List<TableQueryVO.RealTimeDataElement> res = new ArrayList<>();

        for (CrossLaneSnapshotDataDTO dto : dtoList) {
            TableQueryVO.RealTimeDataElement element = new TableQueryVO.RealTimeDataElement();

            String timeStamp = dto.getTimeStamp();
            long timeStampInt = Long.parseLong(timeStamp);
            Date date = new Date(timeStampInt * 1000); // 转换为毫秒
            element.setTime(date);

            element.setDir(dto.getDir());

            String laneId = dto.getLaneId();
            String laneSort = laneId.substring(laneId.length() - 2);
            element.setLaneSort(Integer.valueOf(laneSort));

            double staticQueueLength = dto.getStaticQueueLength();
            int staticQueueLengthRound = (int) Math.round(staticQueueLength);
            element.setQueueLength(staticQueueLengthRound);

            double headCarPosition = dto.getHeadCarPosition();
            Integer headCarPositionRound = (int) Math.round(headCarPosition);
            if (headCarPositionRound == -1) {
                headCarPositionRound = null;
            }
            element.setFirstCarPosition(headCarPositionRound);

            double tailCarPosition = dto.getTailCarPosition();
            Integer tailCarPositionRound = (int) Math.round(tailCarPosition);
            if (tailCarPositionRound == -1) {
                tailCarPositionRound = null;
            }
            element.setEndCarPosition(tailCarPositionRound);
            res.add(element);
        }

        return res;
    }

    @Override
    public TableQueryVO tableRealTime(CrossIdAndIsFirstBO crossIdAndIsFirstBO) throws Exception {
        String crossId = crossIdAndIsFirstBO.getCrossId();
        Integer isFirstInvoke = crossIdAndIsFirstBO.getIsFirstInvoke();
        // 如果是初次调用，需初始化开始时间
        if (1 == isFirstInvoke) {
            initFirstDate();
        }
        // 以当前时间为结束时间
        Date endDate = new Date();
        int endTimeStamp = (int) (endDate.getTime() / 1000);
        int realTimeStartTimeStamp = (int) (tableRealTimeFirstDate.getTime() / 1000);
        int cycleStartTimeStamp = (int) (tableCycleFirstDate.getTime() / 1000);

        return buildTableQueryVO(crossId, cycleStartTimeStamp, endTimeStamp, realTimeStartTimeStamp);
    }

    @NotNull
    private TableQueryVO buildTableQueryVO(
            String crossId, int cycleStartTimeStamp, int endTimeStamp, int realTimeStartTimeStamp)
            throws Exception {

        TableQueryVO tableQueryVO = new TableQueryVO();

        // 查询秒级数据
        List<CrossLaneSnapshotDataDTO> dtoList =
                laneSnapshotDataQueryService.queryByCrossIdAndTimeSpan(crossId, realTimeStartTimeStamp, endTimeStamp);
        tableQueryVO.setRealTimeData(buildRealTimeList(dtoList));

        // 查询周期数据
        List<CrossLaneDataHistPOExt> poExtList = crossLaneDataHistMapper
                .selectByCrossIdAndTimeSpan(crossId, cycleStartTimeStamp, endTimeStamp);
        tableQueryVO.setCycleData(buildCycleData(poExtList));

        return tableQueryVO;
    }

    @Override
    public OptTypeVO optType(CrossIdBO crossIdBO) {
        String crossId = crossIdBO.getCrossId();
        List<CrossSchemeOptLogPO> poList = crossSchemeOptLogMapper.selectMaxByCrossId(crossId);
        CrossSchemeOptLogPO po = poList.get(0);
        String optType = po.getOptType();
        Integer optTypeInt = Integer.valueOf(optType);

        OptTypeVO optTypeVO = new OptTypeVO();
        optTypeVO.setOptType(optTypeInt);
        return optTypeVO;
    }

    private void initFirstDate() {
        tableRealTimeFirstDate = new Date();
        // 将当前时间向前推10分钟
        LocalDateTime now = LocalDateTime.now();
        LocalDateTime tenMinutesAgo = now.minus(10, ChronoUnit.MINUTES);
        Date resultDate = Date.from(tenMinutesAgo.atZone(ZoneId.systemDefault()).toInstant());
        tableCycleFirstDate = resultDate;
    }

    private List<TableQueryVO.CycleDataElement> buildCycleData(List<CrossLaneDataHistPOExt> poExtList) {
        List<TableQueryVO.CycleDataElement> res = new ArrayList<>();

        for (CrossLaneDataHistPOExt po : poExtList) {
            TableQueryVO.CycleDataElement vo = new TableQueryVO.CycleDataElement();

            Integer batchTime = po.getBatchTime();
            // 将10位时间戳转换为毫秒
            long millis = batchTime * 1000L;
            // 使用毫秒值创建Date对象
            Date dateFromTimestamp = new Date(millis);
            vo.setTime(dateFromTimestamp);

            vo.setDir(po.getDir());
            vo.setLaneSort(po.getSort());
            vo.setFlow(po.getFlow());

            Double speed = po.getSpeed();
            vo.setSpeed((int) Math.round(speed));
            Double queueLength = po.getQueueLength();
            vo.setQueueLength((int) Math.round(queueLength));

            Integer delayTime = po.getDelayTime();
            vo.setDelayTime(delayTime);

            Double stopTimes = po.getStopTimes();
            vo.setStopTimes((int) Math.round(stopTimes));
            Double vehheadTime = po.getVehheadTime();
            vo.setVehheadTime((int) Math.round(vehheadTime));

            Double timeOccupancyDouble = po.getTimeOccupancy();
            if (timeOccupancyDouble != null) {
                vo.setTimeOccupancy((int) Math.round(timeOccupancyDouble * 100));
            }

            Double vehicleNumsRatioMeanDoulbe = po.getVehicleNumsRatioMean();
            if (vehicleNumsRatioMeanDoulbe != null) {
                vo.setVehicleNumsRatioMean((int) Math.round(vehicleNumsRatioMeanDoulbe * 100));
            }

            res.add(vo);
        }

        List<TableQueryVO.CycleDataElement> sortedRes = res.stream()
                .sorted(Comparator.comparing(TableQueryVO.CycleDataElement::getTime))
                .collect(Collectors.toList());

        return sortedRes;
    }

    private MainlineSchemeAnalysisVO.GreenwaveData findMatchingData(
            List<MainlineSchemeAnalysisVO.GreenwaveData> greenwaveData) {

        Calendar now = Calendar.getInstance();
        String currentDayType = getDayType(now);
        String currentTime = new SimpleDateFormat("HH:mm").format(now.getTime());

        for (MainlineSchemeAnalysisVO.GreenwaveData data : greenwaveData) {
            String name = data.getName();
            // 旅游路 东向西 转山西路至霞景路路段:正向绿波 工作日 07:00-07:40
            String[] split1 = name.split(" ");
            String dayType = split1[3];

            Pattern pattern = Pattern.compile(".*?(\\d{2}:\\d{2}-\\d{2}:\\d{2})");
            Matcher matcher = pattern.matcher(name);
            if (matcher.matches()) {
                String timeRange = matcher.group(1);
                String[] times = timeRange.split("-");
                try {
                    Date startTime = new SimpleDateFormat("HH:mm").parse(times[0]);
                    Date endTime = new SimpleDateFormat("HH:mm").parse(times[1]);
                    Date currentTimeDate = new SimpleDateFormat("HH:mm").parse(currentTime);

                    if (currentTimeDate.after(startTime) && currentTimeDate.before(endTime)) {
                        if (!dayType.equals("工作日") && !dayType.equals("周末")) {
                            SimpleDateFormat sdf = new SimpleDateFormat("M月d日");
                            String currentDate = sdf.format(now.getTime());

                            if (dayType.equals(currentDate)) {
                                return data;
                            }
                        } else if ((dayType.equals("工作日") && currentDayType.equals("工作日"))
                                || (dayType.equals("周末") && currentDayType.equals("周末"))) {
                            return data;
                        }
                    }
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
        }

        return null;
    }

    private String getDayType(Calendar calendar) {
        int dayOfWeek = calendar.get(Calendar.DAY_OF_WEEK);
        if (dayOfWeek == Calendar.SATURDAY || dayOfWeek == Calendar.SUNDAY) {
            return "周末";
        } else {
            return "工作日";
        }
    }

    private void setMetrics(GreenwaveCrossMetricsVO greenwaveCrossMetricsVO, GreenwaveCrossPO greenwaveCrossPO,
                            List<CrossTurnDataHistPO> listForNoPark, List<CrossTurnDataHistPO> listForPhaseQueue) {
        if (CollectionUtil.isNotEmpty(listForNoPark) && CollectionUtil.isNotEmpty(listForPhaseQueue)) {
            Integer noparkPassRate = getAveRate(listForNoPark);
            greenwaveCrossMetricsVO.setNoparkPassRate(noparkPassRate);
            Integer uncoordinatePhaseQueue = getPhaseQueue(listForPhaseQueue);
            greenwaveCrossMetricsVO.setUncoordinatePhaseQueue(uncoordinatePhaseQueue);
            Double length = greenwaveCrossPO.getNextCrossLen();
            Double speed = getSpeed(listForNoPark);
            double travelTime = length / speed;
            greenwaveCrossMetricsVO.setTrvalTime((int)travelTime);
        }
    }

    private Double getSpeed(List<CrossTurnDataHistPO> listForNoPark) {
        Double total = 0.0;
        for (CrossTurnDataHistPO crossDirDataHistPO : listForNoPark) {
            Double speed = crossDirDataHistPO.getSpeed();
            total += speed;
        }
        double avgSpeed = total / listForNoPark.size();
        return avgSpeed;
    }

    private Integer getPhaseQueue(List<CrossTurnDataHistPO> listForPhaseQueue) {
        double maxQueue = 0.0;
        for (CrossTurnDataHistPO crossDirDataHistPO : listForPhaseQueue) {
            double queueLength = crossDirDataHistPO.getQueueLength();
            if (queueLength > maxQueue) {
                maxQueue = queueLength;
            }
        }
        return (int)maxQueue;
    }

    private Integer getAveRate(List<CrossTurnDataHistPO> crossDirDataHistPOList) {
        Double total = 0.0;
        for (CrossTurnDataHistPO crossDirDataHistPO : crossDirDataHistPOList) {
            Double noStopRate = crossDirDataHistPO.getNoStopRate();
            total += noStopRate;
        }
        int size = crossDirDataHistPOList.size();
        double avg = total / size;
        return (int)(avg * 100);
    }

    private List<GreenWaveCrossMonitorVO> buildRes(List<GreenwaveCrossPO> greenwaveCrossPOList) {
        List<GreenWaveCrossMonitorVO> res = new ArrayList<>();

        // 根据路口ID去重，并按路口顺序号排序
        Map<String, GreenwaveCrossPO> uniqueMap = greenwaveCrossPOList.stream()
                .collect(Collectors.toMap(
                        GreenwaveCrossPO::getCrossId,    // 使用crossId作为键
                        Function.identity(),              // 使用对象本身作为值
                        (existing, replacement) -> existing));  // 如果有重复的key, 保留第一个

        // 从uniqueMap中获取去重后的记录列表
        List<GreenwaveCrossPO> uniqueList = new ArrayList<>(uniqueMap.values());

        // 根据sort属性对uniqueList进行排序
        uniqueList.sort(Comparator.comparingInt(GreenwaveCrossPO::getSort));

        for (GreenwaveCrossPO greenwaveCrossPO : uniqueList) {
            GreenWaveCrossMonitorVO greenWaveCrossMonitorVO = new GreenWaveCrossMonitorVO();

            String crossId = greenwaveCrossPO.getCrossId();
            CrossDataRealtimePO crossDataRealtimePO = crossDataRealtimeMapper.selectByCrossId(crossId);
            BaseCrossInfoPO baseCrossInfoPO = baseCrossInfoMapper.selectById(crossId);
            if (crossDataRealtimePO != null && baseCrossInfoPO != null) {
                greenWaveCrossMonitorVO.setCrossId(crossId);
                greenWaveCrossMonitorVO.setCrossName(baseCrossInfoPO.getName());
                greenWaveCrossMonitorVO.setFlow(crossDataRealtimePO.getFlow());

                List<CrossDirDataRealtimePO> crossDirDataRealtimePOList =
                        crossDirDataRealtimeMapper.selectBycrossId(crossId);
                Integer capacity = 0;
                for (CrossDirDataRealtimePO crossDirDataRealtimePO : crossDirDataRealtimePOList) {
                    capacity += crossDirDataRealtimePO.getCapacity();
                }
                greenWaveCrossMonitorVO.setCapacity(capacity);

                greenWaveCrossMonitorVO.setCongestionIndex(crossDataRealtimePO.getCongestionIndex());
                res.add(greenWaveCrossMonitorVO);
            }
        }
        return res;
    }

    private List<AbnormalCrossDetailVO.DirDataElement> buildDirData(String crossId) {
        initPreStatus();
        List<AbnormalCrossDetailVO.DirDataElement> dirData = new ArrayList<>();
        // 获取当前时间之前40分钟的10位时间戳
        long currentSeconds = DateUtil.currentSeconds();
        long preSeconds = currentSeconds - 40 * 60;
        List<CrossDirDataHistPO> crossDirDataHistPOList =
                crossDirDataHistMapper.selectByCrossIdAndTimestamp(crossId, preSeconds);
        List<Integer> dirs = baseCrossDirInfoMapper.selectInDirsByCrossId(crossId);

        Date currentMinute = DateUtil.date(currentSeconds * 1000);
        Date startMinute = DateUtil.offsetMinute(currentMinute, -59);

        for (int i = 1; i <= 60; i++) {
            // 下一分钟
            Date nextMinute = DateUtil.offsetMinute(startMinute, 1);
            for (Integer dir : dirs) {
                AbnormalCrossDetailVO.DirDataElement dirDataElement = new AbnormalCrossDetailVO.DirDataElement();
                dirDataElement.setDir(dir);
                dirDataElement.setTime(startMinute);
                int status = calcStatus(crossDirDataHistPOList, startMinute, nextMinute, dir);
                dirDataElement.setStatus(status);
                dirData.add(dirDataElement);
            }
            startMinute = nextMinute;
        }

        return dirData;
    }

    private void initPreStatus() {
        for (Integer key : preStatus.keySet()) {
            preStatus.put(key, 1);
        }
    }

    /**
     * 计算路口状态
     *
     * @author Kent HAN
     * @date 2023/6/20 9:07
     */
    private Integer calcStatus(List<CrossDirDataHistPO> crossDirDataHistPOList, Date preMinute, Date currentMinute,
                               int dir) {
        for (CrossDirDataHistPO crossDirDataHistPO : crossDirDataHistPOList) {
            int dirType = crossDirDataHistPO.getDirType();
            long batchTime = crossDirDataHistPO.getBatchTime();
            long preSecond = preMinute.getTime() / 1000;
            long currentSecond = currentMinute.getTime() / 1000;
            if ( dirType == dir && batchTime <= currentSecond && batchTime >= preSecond) {
                Integer status = crossDirDataHistPO.getStatus();
                // 转换状态，3、4都算拥堵
                if (status == 4) {
                    status = 3;
                }
                preStatus.put(dir, status);
            }
        }
        return preStatus.get(dir);
    }

    private List<AbnormalCrossDetailVO.TurnDataElement> buildTurnData(String crossId) {
        List<AbnormalCrossDetailVO.TurnDataElement> turnData = new ArrayList<>();
        List<CrossTurnDataRealtimePO> crossTurnDataRealtimePOList = crossTurnDataRealtimeMapper.selectByCrossId(crossId);
        Map<Integer, List<CrossTurnDataRealtimePO>> collect = crossTurnDataRealtimePOList.stream()
                .collect(Collectors.groupingBy(CrossTurnDataRealtimePO::getInDir));
        for (Map.Entry<Integer, List<CrossTurnDataRealtimePO>> entry : collect.entrySet()) {
            AbnormalCrossDetailVO.TurnDataElement turnDataElement = new AbnormalCrossDetailVO.TurnDataElement();
            Integer dir = entry.getKey();
            turnDataElement.setDir(dir);
            // 构造转向列表
            List<CrossTurnDataRealtimePO> subList = entry.getValue();
            List<AbnormalCrossDetailVO.TurnDataElement.TurnListElement> turnList = buildTurnList(subList);
            turnDataElement.setTurnList(turnList);
            
            turnData.add(turnDataElement);
        }
        return turnData;
    }

    private List<AbnormalCrossDetailVO.TurnDataElement.TurnListElement> buildTurnList(
            List<CrossTurnDataRealtimePO> subList) {
        List<AbnormalCrossDetailVO.TurnDataElement.TurnListElement> turnList = new ArrayList<>();

        for (CrossTurnDataRealtimePO crossTurnDataRealtimePO : subList) {
            AbnormalCrossDetailVO.TurnDataElement.TurnListElement turnListElement =
                    new AbnormalCrossDetailVO.TurnDataElement.TurnListElement();

            turnListElement.setTurn(crossTurnDataRealtimePO.getTurnType());
            turnListElement.setTurnQueueLength(crossTurnDataRealtimePO.getQueueLength());
            turnListElement.setTurnStopTimes(crossTurnDataRealtimePO.getStopTimes());
            turnListElement.setFlow(crossTurnDataRealtimePO.getFlow());

            turnList.add(turnListElement);
        }
        return turnList;
    }

    private static void buildAbnormalCrossStats(AbnormalCrossVO abnormalCrossVO,
                                                List<AbnormalCrossListVO> abnormalCrossListVOList) {
        AbnormalCrossStats abnormalCrossStats = new AbnormalCrossStats();
        Integer normal = abnormalCrossStats.getNormal();
        Integer unbalance = abnormalCrossStats.getUnbalance();
        Integer congestion = abnormalCrossStats.getCongestion();
        Integer spillover = abnormalCrossStats.getSpillover();
        for (AbnormalCrossListVO abnormalCrossListVO : abnormalCrossListVOList) {
            Integer realtimeStatus = abnormalCrossListVO.getRealtimeStatus();
            if (realtimeStatus == 0) {
                normal += 1;
            } else if (realtimeStatus == 1) {
                unbalance += 1;
            } else if (realtimeStatus == 2) {
                congestion += 1;
            } else if (realtimeStatus == 3) {
                spillover += 1;
            }
        }
        abnormalCrossStats.setNormal(normal);
        abnormalCrossStats.setUnbalance(unbalance);
        abnormalCrossStats.setCongestion(congestion);
        abnormalCrossStats.setSpillover(spillover);
        abnormalCrossVO.setAbnormalCrossStats(abnormalCrossStats);
    }

    private static void changeStatus(List<GreenwaveListVO> greenwaveListVOList) {
        if (greenwaveListVOList != null) {
            for (GreenwaveListVO greenwaveListVO : greenwaveListVOList) {
                if (greenwaveListVO.getRealtimeStatus() == 4) {
                    greenwaveListVO.setRealtimeStatus(3);
                }
            }
        }
    }

    private List<GreenwaveListVO.CrossListElement> buildCrossList(Integer greenwaveId) {
        List<GreenwaveListVO.CrossListElement> crossList = new ArrayList<>();
        List<String> crossIdList = greenwaveInfoMapper.selectCrossIdsById(greenwaveId);
        if (CollectionUtil.isNotEmpty(crossIdList)) {
            List<BaseCrossInfoPO> baseCrossInfoPOList = baseCrossInfoMapper.selectByCrossIds(crossIdList);
            for (BaseCrossInfoPO baseCrossInfoPO : baseCrossInfoPOList) {
                GreenwaveListVO.CrossListElement crossListElement = new GreenwaveListVO.CrossListElement();
                crossListElement.setCrossId(baseCrossInfoPO.getId());
                crossListElement.setName(baseCrossInfoPO.getName());
                List<Double> location = buildLocation(baseCrossInfoPO);
                crossListElement.setLocation(location);

                crossList.add(crossListElement);
            }
        }
        return crossList;
    }

    private List<Double> buildLocation(BaseCrossInfoPO baseCrossInfoPO) {
        String location = baseCrossInfoPO.getLocation();
        double[] lonLat = CrossUtil.getLonLat(location);
        List<Double> res = new ArrayList<>();
        res.add(lonLat[0]);
        res.add(lonLat[1]);
        return res;
    }

    private static void buildDuration(GreenwaveListVO greenwaveListVO) {
        Date startTime = greenwaveListVO.getStartTime();
        Date endTime = greenwaveListVO.getEndTime();
        Long duration = null;
        if (endTime != null) {
            duration = DateUtil.between(startTime, endTime, DateUnit.MINUTE);
        } else {
            duration = DateUtil.between(startTime, new Date(), DateUnit.MINUTE);
        }
        greenwaveListVO.setDuration(duration);
    }
}
